#include "may.h"

/*-------------------------------------------------------------------*/
/*
 * This routine crunches numbers.
 * 
 * Actually, all it does is generate a neat Moire pattern.
 */

void filled_circle (flt_pt_data, flt_pt_datalen, xloc, yloc)
float  		flt_pt_data [];
unsigned int	flt_pt_datalen;
float		xloc, yloc;
{
unsigned int	i;
int		ip, jp;

jp = (int) yloc;
for (i=0; i<flt_pt_datalen; i++) {
   ip = (int) xloc + (int) i;
   flt_pt_data [i] = ((float) ((ip*ip+jp*jp) % 2500)) / 2500.0;
   }
}

/*-------------------------------------------------------------------*/
/*
 * This routine crunches numbers.
 *
 * Actually, it computes the escape from one of basic mandelbrot 
 * iterator.  We're just demoing this here because it burns a few more
 * cycles than the routine above.
 */


void brot_out (flt_pt_data, flt_pt_datalen, 
		re_start, im_start, delta, itermax)
float  		flt_pt_data [];
unsigned int	flt_pt_datalen;
double		re_start, im_start, delta;
int		itermax;
/* this routine fills in the exterior of the mandelbrot set using */
/* the classic algorithm */
{
unsigned int	i,j;
int		ip, jp;
double		re_position, im_position;
double		re, im, tmp;
int		loop;

im_position = im_start;
re_position = re_start;
for (i=0; i<flt_pt_datalen; i++) {
   re = re_position;
   im = im_position;
   for (loop=1; loop <itermax; loop++) {
      tmp = re*re - im*im + re_position;
      im = 2.0*re*im + im_position;
      re = tmp;
      if ((re*re + im*im) > 4.0) break;
      }    
   flt_pt_data [i] = ((float) (loop%60)) / 60.0; 
   re_position += delta;
   }
}

/*-------------------------------------------------------------------*/
/*
 * This routine

 * This program implements a very crude algorithm for distributing and 
 * load balancing the work neccessary to compute the the value of pixels in
 * a pixel array. The pixar is subdivided into horizontal strips of size 
 * 1 by PIX_LINE_LEN, which are then shipped off in arbitrary order to
 * other processors running MAY.
 */

#define PIX_WID 1024

void main (argc, argv) 
int argc;
char **argv; 
{
    float	data [PIX_WID];		/* my data array */

    struct	mayProcId	*myself, *myparent;
    struct 	mayMsghead	header;
    int 	messlen;
    int 	retval;		/* returned value */
    struct initial_data {
	unsigned int ix;
	unsigned int iy;
	unsigned int ilen;
	unsigned int pad;	/* sgi machines and word boundries */
        double fx;
	double fy;
	double delta;
	unsigned int itermax;
	int not_done;
	} received_data;

    mayInitTcp ();
    myparent = mayGetParentTcp ();

#ifdef TRACE
    printf ("slave: initialized \n");
#endif
    while (TRUE) {
       header.msgptr = (char *) &received_data;
       /* if we timed out waiting for a message, quit */
       retval = mayReceive (NULL, NULL, NULL, 400, &header, &messlen);
       if ((retval == MAY_TIMED_OUT) || (retval == MAY_NO_OPEN_SOCKS)) break;
       /* if we get message 999, quit */
       if (header.msgtype == 999) break;
       if (messlen != sizeof (struct initial_data)) {
          printf (" slave: wrong size message recvd, got %d shoulda got %d\n",
			messlen, sizeof (struct initial_data) );
          }

#ifdef TRACE
       printf (" got the %d th mess for %d %d \n",  header.subtype,
       received_data.ix, received_data.iy );
#endif

       (void) brot_out (data, received_data.ilen, 
		received_data.fx,     received_data.fy,
		received_data.delta,     (int) received_data.itermax);

     /*   (void) filled_circle (data, received_data.ilen, 
		(float)     received_data.ix, 
		(float)     received_data.iy);  */

       maySend (myparent, (long) header.msgtype, (short) header.subtype,
		(char     *) &data[0], 
		(int)     (received_data.ilen * sizeof (float)) ); 

       }
#ifdef TRACE
   printf ("Goodbye, World");
#endif
}


/*-------------------------------------------------------------------*/
extern double drand48 ();
#define LOOP_COUNT 18

void mandelbrot_in (glob, sizex, sizey,
               re_center, im_center, width, itermax)
float  		glob [];
unsigned int	sizex, sizey;
double		re_center, im_center, width;
int		itermax;
/* this routine fills in the interior of the mandelbrot set using */
/* the classic algorithm */
{
unsigned int	i,j, globlen;
int		ip, jp;
double		re_start, im_start, delta;
double		re_position, im_position;
double		re, im, tmp;
int		loop;
double		x_width, y_width;
double		x_slope, y_slope, x_off, y_off;
int		horiz_pix, vert_pix;


x_width = width;
y_width = width * ((double) sizey) / ((double) sizex);
re_start = re_center - x_width / 2.0;
im_start = im_center - y_width / 2.0;
x_slope = ((double) sizex) / x_width;
y_slope = ((double) sizey) / y_width;
x_off = x_slope * re_start;
y_off = y_slope * im_start;


globlen = sizex*sizey;
for (i=0; i<globlen; i++) glob [i] = 0.0;

/* very crudely, we want as many random seeds as there are pixels */
for (i=0; i<globlen*itermax; i++) {
   im_position = y_width * (drand48() - 0.5) + im_center;
   drand48 (); /* throw one away -- absolutely must do this-- dont ask */
   re_position = x_width * (drand48() - 0.5) + re_center;

   if (i%(itermax*sizex)==0) printf(" start row %d\n", i/(sizex*itermax));

   re = im = 0.0;
   for (loop=0; loop < LOOP_COUNT; loop++) {
      tmp = re*re - im*im + re_position;
      im = 2.0*re*im + im_position;
      re = tmp;
      if ((re*re + im*im) > 4.0) break;
      horiz_pix = (int) (x_slope * re - x_off);
      vert_pix = (int) (y_slope * im - y_off);
      if ( (horiz_pix >= 0) && (horiz_pix < sizex) && 
         (vert_pix >= 0) && (vert_pix < sizey)) {
         glob [vert_pix*sizex + horiz_pix] ++;
         }
      }    
   }

/* renormalize */
for (i=0; i<globlen; i++) {
   glob [i] = glob[i] / (double) (itermax*LOOP_COUNT) ;
   glob [i] = ((double) (((int) glob[i]) %3)) / 3.0;
   }
}
