#include "may.h"
#include "machines.h"

main () {

   /*
    * This program illustrates a sample "parent" or "master" program.
    *
    * This program, being the top-level processes, must be started 
    * by the user.  The "slave" processes will be started at the 
    * request of this program.  This happens automatically, in the sense
    * that the user does not have to do anything special to start
    * the "slave" programs (other than to make sure that they have 
    * been installed, and that the MAY daemon is running on all involved
    * processors)
    *
    * The first thing this program does, after it initializes, is to 
    * start up a slave program that is on the same processor as this
    * master program.  It then listens for, and prints out, a
    * "greetings" message that the slave sends back.
    *
    * Next, it starts up ("creates") a slave process on each of two
    * remote machines. It prepares a data structure to be sent to each
    * of the slaves (the slaves will receive the structure, perform
    * some arithmetic on it, and send the results back).  It sends the
    * data first to one slave, listens for the reply, and prints out
    * the reply.  Changing the data, it then sends it to the second
    * slave, again listens for the reply, and prints out the result.
    *
    * In the notes below, the first person (me, I) is used to describe
    * the actions of this program in respect to the "child" or "slave"
    * programs.
    */


   struct mayProcId *local_childs_addr;	/* This is address of local slave */ 
   struct mayProcId *ibmrt_addr;	/* address of one remote slave */
   struct mayProcId *sgipi_addr;	/* address of another remote slave */

   struct mayMsghead getit; 	/* mandatory MAY header structure */
   char buff [200];		/* buffer for receiving an ASCII string */
   int len;			/* message length */

   struct quiz { 	/* this is a sample data structure that	*/
	int ia;  	/* a typical application might use to  	*/
	int ib; 	/* communicate with the various remote 	*/
	int ians; 	/* processes. Both parent and child must*/
	float a;	/* have a copy of this structure to be	*/
	float b;	/* able to understand one-another.	*/
	float sum;
	double da;
	double db;
	double dasum;
		} quizling;


   /* initialize myself */
   printf (" parent about to mayinit \n");
   (void) mayInitTcp ();


   /*
    * Start up a slave process on the same machine that 
    * I am executing on. The path name is the file location 
    * of the slave process. It must be executable. 
    */
   printf (" parent about to local maycreate \n");

/*
   local_childs_addr = mayCreate (NULL, 
                                  "/src/linas/may/examples/primer/slave.tcp",
                                  MAY_CREATE_TCP);  
*/
   local_childs_addr = mayCreate (LocalMachine, LocalPath,
                                  MAY_CREATE_TCP);  

#ifdef DEBUG
   printf ("dad childs family (2=inet) %d\n", local_childs_addr->address.sin_family);
   printf ("dad childs addr %x\n", local_childs_addr->address.sin_addr.s_addr);
   printf ("dad childs port %d\n", local_childs_addr->address.sin_port);
#endif

   /*
    * receive the ASCII "greeting" message from the local slave. Note
    * some complication because the data area in which the message will
    * be received has to be set up.  Print out the ascii message.
    */
   len = sizeof (buff);
   getit.msgptr = buff;
   mayReceive (local_childs_addr, 1, 0, 3, &getit, &len);
   printf (" This is what I heard from the local child: %s \n", buff);

   /* 
    * Start up slaves on each of two remote machines, called "thehill"
    * and "eldorado".  The path names indicate where, on each of these
    * two respective machines, the slave programs can be found.  Note
    * that the slave programs must have been previously down-loaded to
    * these machines, e.g. by tftp, or by running a distributed file
    * system.  The MAY environment does not currently have any features
    * to automatically down-load slave executables.
    */
   printf (" parent about to remote maycreate \n");
/*
   ibmrt_addr = mayCreate ("thehill", 
                           "/u/linas/src/may/samples/primer/slave",
                            MAY_CREATE_TCP);  
*/
   ibmrt_addr = mayCreate (RTMachine, RTPath, MAY_CREATE_TCP);  

/*
   sgipi_addr = mayCreate ("eldorado", 
                           "/usr/people/linas/may/demos/slave",
                            MAY_CREATE_TCP);  
*/
   sgipi_addr = mayCreate (PIMachine, PIPath, MAY_CREATE_TCP);  

   /* prepare a message to send to one of the slaves, and send it */
   quizling.ia = 5;
   quizling.ib = 6;
   quizling.a = 4.0;
   quizling.b = 3.0;
   quizling.da = -2.718281828e-8;
   quizling.db = 3.14159265358979; 
   len = sizeof (struct quiz);
   maySend (ibmrt_addr, 43, 22, (char *) &quizling, len);

   /* 
    * Receive the results  of the slave's calculations on the data
    * we had just sent above. (Note, the slave had sent this reply 
    * after the greeting which we receive below -- but thats ok --
    * we can "open our mail" in any order that we wish.)
    */
   getit.msgptr = (char *) &quizling;
   len = sizeof (struct quiz);
   mayReceive (ibmrt_addr, 43, 0, 3, &getit, &len);

   /* 
    * Receive the "greeting" message that the remote process sends 
    */
   len = sizeof (buff);
   getit.msgptr = buff;
   mayReceive (ibmrt_addr, 1, 0, 3, &getit, &len);

   /*
    * print out what we have received
    */
   printf (" From a slave running on an IBM PC-RT I heard ");
   printf (" the following greeting: %s \n", buff);
   printf (" I also heard that %d + %d = %d \n", 
                          quizling.ia, quizling.ib, quizling.ians );
   printf (" and I heard sqrt (%f) = %f \n", 
                          quizling.a, quizling.sum);
   printf (" and I that %.18g = %.18g \n", 
                          quizling.db, quizling.dasum);


   /*
    * Now, prepare a different message to be sent to the other slave,
    * and sent it.
    */
   quizling.ia = 4;
   quizling.ib = 3;
   quizling.a = 2.0;
   quizling.b = 3.0;
   quizling.da = -2.718281828e-8;
   quizling.db = 3.14159265358979;
   len = sizeof (struct quiz);
   maySend (sgipi_addr, 43, 22, (char *) &quizling, len);

   /* 
    * Receive the results  of the slave's calculations on the data
    * we had just sent above. 
    */
   getit.msgptr = (char *) &quizling;
   len = sizeof (struct quiz);
   mayReceive (sgipi_addr, 43, 0, 3, &getit, &len);

   /* 
    * Receive the "greeting" message that the remote process sends 
    */
   len = sizeof (buff);
   getit.msgptr = buff;
   mayReceive (sgipi_addr, 1, 0, 3, &getit, &len);

   /*
    * print out what we have received
    */
   printf (" From a slave running on a SGI Personal Iris I heard");
   printf (" the following greeting: %s \n", buff);
   printf (" I also heard that %d + %d = %d \n", 
                          quizling.ia, quizling.ib, quizling.ians );
   printf (" and I heard sqrt (%f) = %f \n", 
                          quizling.a, quizling.sum);
   printf (" and I that %.18g = %.18g \n", 
                          quizling.db, quizling.dasum);

   /* 
    * say bye-bye
    */
   printf (" parent about to exit\n");
   mayTerminate();
}
