#include <math.h>
#include "may.h"

main () {

   /*
    * This program illustrates a sample "child" or "slave" program.
    *
    * This program, like all "child" processes in the MAY environment,
    * is started automatically by the MAY daemon at the parent's request.
    *
    * The first thing it does is to dash off a greeting message to the
    * parent. There is no need to do this in general; this is simply a 
    * sample usage.  It next listens for a data packet, or "message", 
    * from the parent.  In this message are some integer, floating
    * point, and double precision numbers.  It performs some stunningly
    * stunningly complex mathematical computations on these numbers,
    * and sends back the results to the parent.  After doing this
    * precisely once, it exits.
    *
    * In the notes below, the first person (me, I) is used to describe
    * the actions of this program in respect to the "parent" or "master"
    * program.
    */


   struct mayProcId *rents_addr;	/* This is my mom's address */

   struct mayMsghead getit; 	/* mandatory MAY header structure */
   int len;			/* message length */

   struct quiz { 	/* this is a sample data structure that	*/
	int ia;  	/* a typical application might use to  	*/
	int ib; 	/* communicate with the various remote 	*/
	int ians; 	/* processes. Both parent and child must*/
	float a;	/* have a copy of this structure to be	*/
	float b;	/* able to understand one-another.	*/
	float sum;
	double da;
	double db;
	double dasum;
		} quizling;


   printf ("Hello, world! \n");
   (void) mayInitTcp ();

   /* Figure out who fired me up  -- i.e. get the network address of
    * my parent */
   rents_addr = mayGetParentTcp ();

#ifdef DEBUG
   printf ("slave rent family (2=inet) %d\n", rents_addr->address.sin_family);
   printf ("slave rent addr %x\n", rents_addr->address.sin_addr.s_addr);
   printf ("slave rent port %d\n", rents_addr->address.sin_port);
#endif
 
   /* send a welcome message back to my parent */
   maySend (rents_addr, 
            1, 1, 
            "greetings from Linas",
            sizeof ("greetings from Linas"));


   /* set up the data structure in which I'll receive data, 
    * and then listen */
   getit.msgptr = (char *) &quizling;
   len = sizeof (struct quiz);
   mayReceive (NULL, 43, 0, 30, &getit, &len);

   /* perform some stunningly complex math */
   quizling.ians = quizling.ia + quizling.ib;
   quizling.sum = sqrt (quizling.a);
   quizling.dasum = 2.0 * acos (0.0);

   /* ... and sent the result of it back home */
   len = sizeof (struct quiz);
   maySend (rents_addr, 43, 22, (char *) &quizling, len);

   /* say bye-bye */
   mayTerminate();

   }
