
#include <stdio.h>
#include <string.h>
#include "may.h"

#define LINE_LENGTH 255

#define CHECK_FOR_NULL(next_tok)			\
{							\
        if (next_tok == NULL) continue;			\
        if (strncmp (next_tok, "#", 1) == 0) continue;	\
        if (strncmp (next_tok, "!", 1) == 0) continue;	\
}


/* =================================================================== */

#ifdef ANSI_C
void get_mandelbrot_stuff (double *re_center, 
                           double *im_center,	/* center of data array */
                           double *width,	/* width of data aray */
                           unsigned int *itermax,  /* max iteration count */
                           unsigned int *rollover)		
#else
void get_mandelbrot_stuff ( re_center, im_center, width, itermax, rollover) 
double	*re_center, *im_center;	/* center of data array */
double	*width;			/* width of data aray */
unsigned int *itermax;		/* max iteration count */
unsigned int *rollover;		
#endif
{
    FILE *fyle;
    char input_record [LINE_LENGTH];
    char *next_tok;

    /* open a file, read data from it and return */
    if ((fyle = fopen ("./mandelbrot", "r")) == NULL) {
        perror ("file open failure: can't find: mandelbrot");
        exit (EXIT_FAILURE);
    }
    /* loop forever, until we break out of loop */
    while (TRUE) {
        /* if end of file or read error, return */
        if (!fgets (input_record, LINE_LENGTH, fyle)) {
           fclose (fyle);
           return;
        }
        /* if input line longer than buffer, ignore it. */
        if (strlen (input_record) == LINE_LENGTH - 1) {
           perror ("file read failure: line too long: machines");
           continue;
        }

        /* ignore comment cards */
        if (strncmp (input_record, "#", 1) == 0) continue;
        if (strncmp (input_record, "!", 1) == 0) continue;

        next_tok = strtok (input_record, " ,;\t\n\r");
        CHECK_FOR_NULL (next_tok);
        *re_center = atof (next_tok);

        next_tok = strtok (NULL, " ,;\t\n\r");
        CHECK_FOR_NULL (next_tok);
        *im_center = atof (next_tok);

        next_tok = strtok (NULL, " ,;\t\n\r");
        CHECK_FOR_NULL (next_tok);
        *width = atof (next_tok);

        next_tok = strtok (NULL, " ,;\t\n\r");
        CHECK_FOR_NULL (next_tok);
        *itermax = atoi (next_tok);

        next_tok = strtok (NULL, " ,;\t\n\r");
        CHECK_FOR_NULL (next_tok);
        *rollover = atoi (next_tok);

printf (" read: %e %e %e %d %d \n", *re_center, *im_center, 
              *width, *itermax, *rollover);

    }

/*
    *re_center = -1.0455124537217;
    *im_center = 0.3264935985845;
    *width = 2.0e-12;
    *itermax = 600;
*/

}

/* ========================= END OF FILE ================================ */
