
/*
Revision History:
15 June 1989 by Linas Vepstas -- insert an XInstallColormap 

24 October 1989 by Linas Vepstas -- if all colormap entries already
                   taken, search for color closest to desired color 

11 January 1990 by Linas Vepstas -- above change defined out,
                   appearently this was an X server bug.  Since some
                   servers may still have this bug, the code was not
                   totally thrown away.

 */

#include "sysdep.h"

#include <math.h>
#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include "debug.h"
#include "extern.h"

typedef struct rgb {
	char r, g, b;
        };

void add_block_to_window();

/*-------------------------------------------------------------------*/

/*
 * Initialize X, create windows, compute color table
 */

initialize_RGB_window (argc, argv, pix_width, pix_height)
int	argc;
char 	**argv;
unsigned int	pix_width, pix_height;	/* size of window/pixmap */
{
    char comp[256];            /* gamma correction table */
    struct rgb map[256];       /* map from dither indices to colors */
    Colormap cmap;             /* a colormap to use */
    int black, white;          /* black, white pixel values */
    Window root;
    XWMHints wm_hints;
    XSizeHints size_hints;
    XColor cur_map[256];
    int i, j, n, bomb;

    width = pix_width;
    height = pix_height;

    /* open display etc. */
    dpy = XOpenDisplay(dpy_name);
    if (!dpy) {
        perror ("Can't Open Display");
        exit (EXIT_FAILURE);
    }
    root = RootWindow(dpy, DefaultScreen(dpy));
    cmap = DefaultColormap(dpy, DefaultScreen(dpy));
    XInstallColormap (dpy, cmap);    /*  Server problem ???? */
    black = BlackPixel(dpy, DefaultScreen(dpy));
    white = WhitePixel(dpy, DefaultScreen(dpy));

    gc = XCreateGC(dpy, root, 0, NULL);
    XSetGraphicsExposures(dpy, gc, 0);

    /* win */
    win = XCreateSimpleWindow(dpy, root, 0, 0, width, height, 3, black, white);
    size_hints.flags = PSize;
    size_hints.width = width;
    size_hints.height = height;
    XSetStandardProperties(dpy, win, "pixels", "pixels", NULL,
        argv, argc, &size_hints);
    XSelectInput(dpy, win, ExposureMask|ResizeRedirectMask);
    XMapRaised(dpy, win);

    {
        /* Need to do this to get window resize to work */
        XSetWindowAttributes swin_attr; 

        swin_attr.override_redirect = TRUE;
        XChangeWindowAttributes (dpy, win, CWOverrideRedirect, &swin_attr);
    }

    /* fill in the color table */
    fprintf (stderr, "...");
    n = ColorMap(map);
    /* Set up a compensating gamma correction map */
    for (i=0; i<256; i++)
        comp[i] = sqrt((i)/256.0)*256.0*gcp + i*(1.0-gcp);
   /* allocate as many colors as possible */
   bomb = FALSE;
   for (i=0; i<n; i++) {
        XColor color;
        color.red = comp[map[i].r] << 8;
        color.green = comp[map[i].g] << 8;
        color.blue = comp[map[i].b] << 8;
        TRACE(("request color %d %d %d ", color.red, color.blue, color.green));
        /* If XAllocColor finds an empty colormap slot, it fills it and
         * returns true.  If it cannot find an empty slot, it returns
         * the nearest slot and returns FALSE. Even if it returns false,
         * it should have done a "lower manhattan" search for best fit.
         */
        if (!XAllocColor(dpy, cmap, &color)) {
            TRACE(("XAllocColor bombed "));
            table[i] = -1;
            bomb = TRUE;
        } else {
            table[i] = color.pixel;
        }
        TRACE(("got color %d %d %d %d\n", color.pixel, color.red,
        color.blue, color.green));
    }

    /* if unable to satisfy all color requests */
    /* perform a least lower manhattan distance search */
    if (bomb) fprintf(stderr, "\nWarning: sharing non-optimal color table ...");
    fprintf(stderr, "\n");

    /* allocate server-side pixmaps */
    pixmap = XCreatePixmap(dpy, win, width, height, 8);

}

/*-------------------------------------------------------------------*/

void add_RGB_block_to_window (block, block_width, block_height,
			x_offset, y_offset)
struct rgb	*block;
unsigned int	block_width;
unsigned int	block_height; 
unsigned int	x_offset, y_offset; /* where to start writing */

{
   char * pixblock;
   unsigned int i, numb;

   numb = block_width * block_height;
   pixblock = (char *) malloc (numb * sizeof(char));

   /* dither, being careful to align properly in the dither matrix */
   DitherAligned (block, block_width, block_height, 
                x_offset, y_offset, pixblock);

   /* pass the results thorugh the color table */
   for (i=0; i<numb; i++) {
      pixblock[i] = table [ pixblock[i] ];
      }

   add_block_to_window (pixblock, block_width, block_height,
			x_offset, y_offset);
   free (pixblock);
}

/*-------------------------------------------------------------------*/

void add_block_to_window (block, block_width, block_height,
			x_offset, y_offset)
char	block [];
unsigned int	block_width;
unsigned int	block_height; 
unsigned int	x_offset, y_offset; /* where to start writing */

{
    /* this routine places a block of pixels into a window.
     * The block is assumed to consist of contiguous 8-bit
     * indeces into the current color table.
     * That is, the size of the array block is assumed to be
     * block_width * block_height * sizeof(char)
     */
    Visual *visual = DefaultVisual(dpy, DefaultScreen(dpy));
    XImage *image, *iimage;
    register int i, j;
    register char *p;

    image = XCreateImage(dpy, visual, 8, ZPixmap, 0, block,
            block_width, block_height, 8, 0);

    /* put our image data into the server-maintained pixmap */
    XPutImage(dpy, pixmap, gc, image, 0, 0, x_offset, y_offset, 
		block_width, block_height);
    /* copy from server-maintained pixmap into the window */
    XCopyArea(dpy, pixmap, win, gc, x_offset, y_offset, 
		block_width, block_height, x_offset, y_offset);

    XDestroyImage (image);	/* free image data structure */
}

/*-------------------------------------------------------------------*/
void check_events () {

    while (XPending(dpy)) dispatch_event();
}

/*-------------------------------------------------------------------*/
/*
 * X event loop
 */

io_error()
{
    exit(0);
}

event_loop()
{
    if (fork()) return;    /* detatch child process - parent returns */
    XSetIOErrorHandler(io_error);

    while (1) dispatch_event();
}

/*-------------------------------------------------------------------*/

dispatch_event()
{
    XEvent event;
    int x, y, wid, hig;

    XNextEvent(dpy, &event);

    switch (event.type) {
    case Expose:
        x = event.xexpose.x;
        y = event.xexpose.y;
        wid = event.xexpose.width;
        hig = event.xexpose.height;
        if (event.xexpose.window==win) {
            XCopyArea(dpy, pixmap, win, gc, x, y, wid, hig, x, y);
            XFlush (dpy);
        } 
        break;

    default:
        break;
    }
}

/*-------------------------------------------------------------------*/

char * Malloc (n)
int n;
{
    char * m = malloc(n);
    if (!m) {
        perror ("can't malloc");
        exit (EXIT_FAILURE);
    } 
    return m;
}

/* ====================== END OF FILE ======================================*/

