/*
 * Modified 4 May 1990 by Linas Vepstas -- to add DitherAlign.
 *   This routine starts off in the right spot in the dither matrix.
 */


#include <stdio.h>


struct rgb {
    char r, g, b;
};


/*
 * Constants
 */

#define R 5                /* number of red shades */
#define G 9                /* number of green shades */
#define B 5                /* number of blue shades */

#define DX 4               /* x dimension of dither matrix */
#define DY 4               /* y dimension of dither matrix */
#define D (DX*DY)          /* number of entries in dither matrix */


/*
 * MIX(r,g,b) calculates the color table index for r,g,b;
 * 0<=r<R, 0<=g<G, 0<=b<B.
 */

#define MIX(r,g,b) (((r)*G+(g))*B+(b))


/*
 * DITH(C,c,d) dithers one color component of one pixel;
 *
 * C is number of shades of the color; C = R, G, or B
 * c is the input pixel, i.e. c = in->r, in->g, or in->b; 0<=c<C
 * d is a dither matrix entry scaled by 256; 0<=d<=(D-1)*256
 *
 * Hint: dithering produces D*(C-1)+1 shades of color C
 */

#define DITH(C,c,d) (((unsigned)((D*(C-1)+1)*c+d)) / (D*256))


/*
 * The dither matrix
 */

static short matrix[DY][DX] = {
     0 *256,  8 *256,  2 *256, 10 *256,
    12 *256,  4 *256, 14 *256,  6 *256,
     3 *256, 11 *256,  1 *256,  9 *256,
    15 *256,  7 *256, 13 *256,  5 *256
};


/*
 * Dither an image
 */

Dither(in, width, height, out)
struct rgb *in;
int width, height;
char *out;
{
    int y, i, j;
    int x, d;
    short *m;

    for (y=0; y<height; y++) {
        m = &(matrix[y&(DY-1)][0]);
        for (x=width; --x>=0; in++) {
            d = m[x&(DX-1)];
            *out++ = MIX(DITH(R,in->r,d), DITH(G,in->g,d), DITH(B,in->b,d));
        }
    }
}

/* DitherAligned starts off by an offset in the dither matrix. */

DitherAligned (in, width, height, xoffset, yoffset, out)
struct rgb *in;
int width, height;
char *out;
{
    int y, i, j;
    int x, d;
    short *m;

    for (y=yoffset; y<yoffset+height; y++) {
        m = &(matrix[y&(DY-1)][0]);
        for (x=xoffset+width; --x>=xoffset; in++) {
            d = m[x&(DX-1)];
            *out++ = MIX(DITH(R,in->r,d), DITH(G,in->g,d), DITH(B,in->b,d));
        }
    }
}


#define diffuse(x) { \
    x += (e[0].x*7 + f[0].x + f[1].x*5 + f[2].x*3) / 16; \
    if (x<0) x = 0;  if (x>255) x = 255; \
}

Diffuse(in, width, height, out)
struct rgb *in;
int width, height;
char *out;
{
    int x, y, i;
    static struct irgb {int r, g, b;} ee[2000], ff[2000], *temp;
    struct irgb *e = ee, *f = ff;

    for (y=0; y<height; y++) {

        for (x=width; --x>=0; in++, e++, f++) {

            int r=in->r, g=in->g, b=in->b;
            int u, v, w;

            diffuse(r);
            diffuse(g);
            diffuse(b);

            u = (r+256/R/2)*R/256;
            v = (g+256/G/2)*G/256;
            w = (b+256/B/2)*B/256;

            e[1].r = r - u*255/(R-1);
            e[1].g = g - v*255/(G-1);
            e[1].b = b - w*255/(B-1);

            *out++ = MIX(u,v,w);
        }

        temp = e-width, e = f-width, f = temp;
    }
}


/*
 * Fill in a color map
 */

ColorMap(map)
struct rgb map[256];
{
    int r, g, b, i;
    for (r=0; r<R; r++) {
        for (g=0; g<G; g++) {
            for (b=0; b<B; b++) {
                i = MIX(r,g,b);
                map[i].r = r*255/(R-1);
                map[i].g = g*255/(G-1);
                map[i].b = b*255/(B-1);
            }
        }
    }
    return R*G*B;
}


/*
 * Change the dither matrix
 */

InitDither(name)
char *name;
{
    int i, j, d;
    FILE *file = fopen(name, "r");
    if (!file)
       return;  /* actually, should print error messg */
    for (i=0; i<DY; i++) {
        for (j=0; j<DX; j++) {
            fscanf(file, "%d", &d);
            matrix[i][j] = 256 * d;
        }
    }
}


