
/* =================================================================
 *
 * Copyright (C) 1989, 1990, 1993 Linas Vepstas
 *
 *                   All Rights Reserved
 *
 * Permission to use, copy, and modify this software and its 
 * documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies 
 * and that both the copyright notice and this permission notice 
 * appear in supporting documentation.
 *
 * Permission to redistribute this software and its documentation
 * is hereby granted, provided that such redistribution is not 
 * performed for monetary profit or gain.
 *
 * Redistribution for profit without express written consent is
 * prohibited.  Contact the author if you wish to use this software 
 * and/or its documentation in a commercial product.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY OF ANY KIND.  
 * THE AUTHOR DISCLAIMS ALL EXPRESS AND IMPLIED WARRANTIES, INCLUDING 
 * BUT NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY AND 
 * FITNESS FOR A PARTICULAR PURPOSE.
 * 
 * IN NO EVENT SHALL THE AUTHOR BE HELD LIABLE FOR ANY SPECIAL, DIRECT, 
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING 
 * IN LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, 
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION 
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Bug reports or fixes may be sent to the author, who may or may not 
 * act upon them as he desires.
 * 
 * The author's last known address is:
 * 
 *                       Linas Vepstas
 *                     1518 Enfield Road
 *                  Austin, Texas 78703-3424
 *                      1-(512)-499-8246
 *                    linas@austin.ibm.com
 *
 * =================================================================
 */

/*
 *	This file contains the necessary definitions to write
 *	distributed programs using U-may on TCP-IP and UNIX sockets.
 *	
 *	Included in /usr/lib/libmay.a are the following routines:
 *
 *	mayinit()
 *	maycreate()
 *	maysend()
 *	mayreceive()
 *	mayterminate()
 */
#include "sysdep.h"
#include "patchlevel.h"

/* ====================================================== */

#define MAY_GENERAL_ERROR	-1   /* unidentified error condition */
#define MAY_TRUNCATED_MSG 	-2   /* incomplete message recv'd */
#define MAY_NO_MEM	 	-3   /* out of memory error */
#define MAY_CLOSE_CONNECTION 	-4   /* socket died */
#define MAY_TIMED_OUT		-5   /* timeout expired */
#define MAY_NO_OPEN_SOCKS	-6   /* all sockets are dead */
#define MAY_INCOMPLETE_READ	-7   /* read not finished */


#define MAY_CREATE_TCP	15  /* default connection should be TCP */
#define MAY_CREATE_UDP	16  /* default connection should be UDP */

/* ====================================================== */

/* structure of machine/process id's */

struct mayProcId {
        struct 	mayProcId *next;	/* not used */
	int	proto;			/* TCP or UDP */
	int	socket_id;		/* socket descriptor */
	struct sockaddr_in address;	/* IP address */
        char 	*hostname;		/* not used */
	int	unread_bytes_remaining;	/* used in TCP reads */
	int 	bytes_read;		/* used in TCP reads */
        char	*partial_msg;		/* used in TCP reads */
        int	is_header_read;		/* used in TCP reads */
	};

struct mayMsghead {
	struct mayProcId sproc;	/* id of the sending process */
	long 	msgtype;	/* message type */
	short 	subtype;
	long 	msglen;		/* equal to sizeof (message) */
	char 	*msgptr;	/* pointer to arbitrary length message */
        long	queue_len;	/* current length of input queue of sender */
	};

/* ==================================================== */
/*
 * extern declarations
 */

#ifdef ANSI_C

extern struct mayProcId *mayCreate (char *requested_host_name,
                                    char *requested_program_name,
                                    int requested_connection_type);

extern void mayCreateFarm (int num_machines,  
                    char *machine_name[],
                    char *file_name[],  
                    int connection_type);


extern struct mayProcId *mayGetParentTcp (void);
extern struct mayProcId *mayGetParentUdp (void);
extern        void      mayInitTcp (void);
extern struct mayProcId *mayInitUdp (void);

extern int mayProcFarm (Void * work,
                int num_pieces, 
                int piecesize, 
                int stride, 
                int ((*callback)()));

extern int mayReceive (struct mayProcId *sproc_ptr,
                       long            msg_type,
                       short           sub_type,
                       unsigned int    timeout,
                       struct mayMsghead   *mess,
                       int             *messlen);

extern int maySend (struct mayProcId *procid,
                    long message_type,
                    short sub_type,
                    void * message,
                    int length);

extern        void      mayTerminate (void);

extern int get_machines_and_file_paths (char **machine_name[],
                                 char **file_name[]);
#else

extern struct mayProcId *mayCreate ();
extern void mayCreateFarm ();
extern struct mayProcId *mayGetParentTcp ();
extern struct mayProcId *mayGetParentUdp ();
extern        void      mayInitTcp ();
extern struct mayProcId *mayInitUdp ();
extern int mayProcFarm ();
extern        int       mayReceive ();
extern        int       maySend ();
extern        void      mayTerminate ();

extern int get_machines_and_file_paths ();

#endif

/* ================ END OF FILE ===================== */
