/* =================================================================
 *
 * Copyright (C) 1989, 1990, 1993 Linas Vepstas
 *
 *                   All Rights Reserved
 *
 * Permission to use, copy, and modify this software and its 
 * documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies 
 * and that both the copyright notice and this permission notice 
 * appear in supporting documentation.
 *
 * Permission to redistribute this software and its documentation
 * is hereby granted, provided that such redistribution is not 
 * performed for monetary profit or gain.
 *
 * Redistribution for profit without express written consent is
 * prohibited.  Contact the author if you wish to use this software 
 * and/or its documentation in a commercial product.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY OF ANY KIND.  
 * THE AUTHOR DISCLAIMS ALL EXPRESS AND IMPLIED WARRANTIES, INCLUDING 
 * BUT NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY AND 
 * FITNESS FOR A PARTICULAR PURPOSE.
 * 
 * IN NO EVENT SHALL THE AUTHOR BE HELD LIABLE FOR ANY SPECIAL, DIRECT, 
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING 
 * IN LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, 
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION 
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Bug reports or fixes may be sent to the author, who may or may not 
 * act upon them as he desires.
 * 
 * The author's last known address is:
 * 
 *                       Linas Vepstas
 *                     1518 Enfield Road
 *                  Austin, Texas 78703-3424
 *                      1-(512)-499-8246
 *                    linas@austin.ibm.com
 *
 * =================================================================
 */


#include "may.h"
#include "maymsg.h"

/* ======================================================== */
/* 
 * maySend
 * Sends message to process
 *
 * errno should be checked for EMSGSIZE, in case the user attempt to
 * send an excessively large message (that is not supported by the
 * underlying protocol).
 */

#ifdef ANSI_C
int maySend (struct mayProcId *procid,
             long message_type,
             short sub_type,
             void * message,
             int length)
#else
int maySend (procid, message_type, sub_type, message, length)
struct mayProcId *procid;
long message_type;
short sub_type;
Void * message;
int length;
#endif
{
   char 	buff [2048];
   struct 	general_msg_hdr 	*hdr_ptr;
   int sent_len;

   /* fill in the header */
   hdr_ptr = (struct general_msg_hdr *) buff;
   hdr_ptr -> msgtype = message_type;
   hdr_ptr -> subtype = sub_type;
   hdr_ptr -> msglen = length + sizeof (struct general_msg_hdr);
   hdr_ptr -> queue_len = may_msg_list_length;
   /* header contains originators (from:) address */
   memcpy (&hdr_ptr->sproc.address, &procid->address, SIZEOSOX);

   /* copy in the message after the header */
   memcpy (&buff [sizeof (struct general_msg_hdr)], message, length);

#ifdef DEBUG
   printf (" maySend sending to sockfd %d ", procid -> socket_id);
   printf ( " addr %d 0x%x %d ", 
      procid -> address.sin_family,
      procid -> address.sin_addr.s_addr, 
      procid -> address.sin_port);
   printf (" mess type %d subtype %d \n", message_type, sub_type);
   printf (" maySend zero fields 0x%x 0x%x \n", 
      procid -> address.sin_zero[0],
      procid -> address.sin_zero[4]);
   printf (" maySend message length (incl. header) is %d \n", hdr_ptr->msglen);
   printf (" maySend head size is %d \n", sizeof (struct general_msg_hdr));
#endif

   /* send the message */
   sent_len = sendto (procid -> socket_id, buff, hdr_ptr->msglen, 0, 
                      &procid->address, SIZEOSOX);
   if (sent_len < 0) {
      perror ("maySend error: sendto");
      return (MAY_GENERAL_ERROR);
   }

   /* return length of message actually sent */
   sent_len -= sizeof (struct general_msg_hdr);
   if (sent_len < 0) sent_len = MAY_TRUNCATED_MSG;
   return (sent_len);

}

/*---------------------- end of file --------------------------*/
