/* =================================================================
 *
 * Copyright (C) 1989, 1990, 1993 Linas Vepstas
 *
 *                   All Rights Reserved
 *
 * Permission to use, copy, and modify this software and its 
 * documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies 
 * and that both the copyright notice and this permission notice 
 * appear in supporting documentation.
 *
 * Permission to redistribute this software and its documentation
 * is hereby granted, provided that such redistribution is not 
 * performed for monetary profit or gain.
 *
 * Redistribution for profit without express written consent is
 * prohibited.  Contact the author if you wish to use this software 
 * and/or its documentation in a commercial product.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY OF ANY KIND.  
 * THE AUTHOR DISCLAIMS ALL EXPRESS AND IMPLIED WARRANTIES, INCLUDING 
 * BUT NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY AND 
 * FITNESS FOR A PARTICULAR PURPOSE.
 * 
 * IN NO EVENT SHALL THE AUTHOR BE HELD LIABLE FOR ANY SPECIAL, DIRECT, 
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING 
 * IN LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, 
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION 
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Bug reports or fixes may be sent to the author, who may or may not 
 * act upon them as he desires.
 * 
 * The author's last known address is:
 * 
 *                       Linas Vepstas
 *                     1518 Enfield Road
 *                  Austin, Texas 78703-3424
 *                      1-(512)-499-8246
 *                    linas@austin.ibm.com
 *
 * =================================================================
 */


#include "may.h"
#include "maymsg.h"

extern struct hostent *gethostbyname();

/* ====================================================== */
/*
 * mayInitTcp 
 * Initialize the may system for a TCP connection
 *
 * Currently, a NOOP
 */

#ifdef ANSI_C
void mayInitTcp (void)
#else
void mayInitTcp ()
#endif

{}

/* ====================================================== */
/*
 * mayGetParentTcp 
 * Return parent's address
 *
 * mayGetParentTcp is a call that is valid only when made by the child.
 * It returns the stream socket address by which it is connected to the
 * parent.  
 *   
 * Note that the socket has already been opened and connected by
 * the local maydaemon.  The local daemon forks and execv's, and due to the
 * nature of execv, the socket descriptor (which should be number 5) 
 * remains open.  Thus, all this routine really
 * has to do is to get a maydaemon service port and address, and get the
 * socket id (in older version of this code, we assumed it was 5.  Turns
 * out, some operating systems do not respect socket numbering;
 * therefore, we need to read the correct socket id from a file.)
 * 
 */

#ifdef ANSI_C
struct mayProcId * mayGetParentTcp (void)
#else
struct mayProcId * mayGetParentTcp ()
#endif
{
   
   struct 	mayProcId	*parent;
   struct 	mayProcId	*tmp;
   struct 	mayProcId	*os;

   /* get prototype template for address */
   tmp = mayGetHosts (NULL, "maydaemon");

   /* allocate space for the list of known connections */
   os = (struct mayProcId *) malloc (SIZEOPRO);
   if (os == NULL) {
      perror ("mayGetParentTcp: error on malloc");
      return(NULL);
   }
   memcpy (os, tmp, SIZEOPRO);
   os->unread_bytes_remaining = 0;
   os->partial_msg = NULL;
   os->next = open_socks;
   open_socks = os;

   /* although we forked off the daemon, we cannot make the
    * assumption that the socket fd is 5. (os->socket_id = 5;) 
    * Instead, read it from a file. */
   mayGetId (&os->socket_id);
   if (os->socket_id == 0) {
      perror ("mayGetParentTcp: error on get socket id");
      return(NULL);
   }

   os->proto = SOCK_STREAM;   /* tcp connection */

   /* allocate space for the thing that will be returned */
   parent = (struct mayProcId *) malloc (SIZEOPRO);
   if (parent == NULL) {
      perror ("mayGetParentTcp: error on malloc");
      return(NULL);
   }
   memcpy (parent, os, SIZEOPRO);
   parent->unread_bytes_remaining = 0;
   parent->partial_msg = NULL;

   /* return hostname, but it must be in a separate data area */
   if ((parent->hostname = 
      (char *) malloc ( strlen(os->hostname)+1 )) == NULL) {
      perror ("mayGetParentTcp: error on malloc");
      return (NULL);
   }
   strcpy (parent->hostname, os->hostname);

   return (parent);
}

/* ====================== END OF FILE ==========================*/
